<?php

require_once 'Vidlinkr/Api.php';
require_once 'Vidlinkr/Exception.php';

class Vidlinkr
{

	const WP_PLUGIN_DIR					= 'vidlinkr';
	const WP_OPTION_CONFIG				= 'coull_vidlinkr_config';

	const FORM_KEY_EMAIL				= 'coull_vidlinkr_email';
	const FORM_KEY_PASSWORD				= 'coull_vidlinkr_password';

	const CONFIG_KEY_PRIVATE_API_KEY	= 'private_api_key';
	const CONFIG_KEY_WEBSITE_ID			= 'website_id';
	const CONFIG_KEY_WEBSITE_URL		= 'website_url';
	const CONFIG_KEY_PUBLISHER_ID		= 'publisher_id';

	private static $_aConfig;

	private static $_sSuccessMessage;
	private static $_sErrorMessage;

	public static function init ()
	{
		add_action('admin_menu', array(__CLASS__,'addOptionsPage'));
		add_filter('the_content', array(__CLASS__, 'swapUrls'));
	}

	public static function addOptionsPage ()
	{
		add_options_page('Vidlinkr', 'Vidlinkr', 'manage_options', 'vidlinkr', array(__CLASS__,'renderOptionsPage'));
		add_filter('plugin_action_links', array(__CLASS__,'renderPluginActions'), 10, 2 );
	}

	public static function renderPluginActions ($aLinks, $sFile)
	{

		if (false !== strpos($sFile, self::WP_PLUGIN_DIR)) {
			array_push($aLinks, sprintf('<a href="%s">Settings</a>', 'options-general.php?page=' . self::WP_PLUGIN_DIR));
		}

		return $aLinks;

	}

	/**
	* Return a URL for the current website installation
	*
	* @return string
	*/
	public static function getWebsiteUrl ()
	{
		return 'http://' . $_SERVER['HTTP_HOST'] . '/';
	}

	/**
	* Handle any POST updates and then include the admin page
	*/
	public static function renderOptionsPage ()
	{

		if (false == empty($_POST)) {
			self::_handleUpdates();
		}

		include 'Vidlinkr/Wp/admin_page.php';

	}

	public static function _handleUpdates ()
	{

		try {

			if (true == isset($_POST[self::FORM_KEY_EMAIL]) && true == isset($_POST[self::FORM_KEY_PASSWORD])) {

				/**
				* Look up private API key from user login details
				*/
				$sPrivateApiKey = self::_getPrivateApiKeyFromLogin($_POST[self::FORM_KEY_EMAIL], $_POST[self::FORM_KEY_PASSWORD]);

			}

			if (true == isset($_POST[self::CONFIG_KEY_PRIVATE_API_KEY])) {

				/**
				* Private API key provided in POST
				*/
				$sPrivateApiKey = $_POST[self::CONFIG_KEY_PRIVATE_API_KEY];

				/**
				* If the API key is not empty, verify it. An exception will be thrown if this fails
				*/
				if (false == empty($sPrivateApiKey)) {
					self::_verifyPrivateApiKey($sPrivateApiKey);
				}

			}

			if (false == isset($sPrivateApiKey)) {

				/**
				* No private API key was provided
				*/
				return;

			}


			/**
			* An API key is present in POST
			*/

			if (0 === strcmp($sPrivateApiKey, self::getConfig(self::CONFIG_KEY_PRIVATE_API_KEY))) {

				/**
				* The provided API key is the same as the one currently stored
				*/

				return;

			}

			/**
			* The provided API key is different from the one currently stored
			*/

			if (true  == empty($sPrivateApiKey)) {

				/**
				* The provided API key is empty. Reset the config.
				*/

				self::setConfig(self::CONFIG_KEY_PRIVATE_API_KEY, null);
				self::setConfig(self::CONFIG_KEY_WEBSITE_ID, null);
				self::setConfig(self::CONFIG_KEY_WEBSITE_URL, null);
				self::setConfig(self::CONFIG_KEY_PUBLISHER_ID, null);

				/**
				* Store config. An exception is thrown if this fails
				*/
				self::storeConfig();

				self::_setSuccessMessage('Your settings were cleared');

				return;

			}

			/**
			* The provided API key is not empty. Assume it has been verified above.
			*/

			/**
			* Store the API key in local config
			*/
			self::setConfig(self::CONFIG_KEY_PRIVATE_API_KEY, $sPrivateApiKey);

			$sWebsiteUrl = self::getWebsiteUrl();

			/**
			* Fetch the details for the website, if it already exists,
			* or add it as a new website.
			*/
			if (false == self::_getWebsite($sWebsiteUrl)) {
				self::_addWebsite($sWebsiteUrl);
			}

			/**
			* Store config. An exception is thrown if this fails
			*/
			self::storeConfig();

			self::_setSuccessMessage('Your settings were updated successfully');

		} catch (Vidlinkr_Exception $oE) {

			self::_setErrorMessage($oE->getMessage());

		}

	}

	/**
	* Verify the provided private API key, returning true if it is valid,
	* or throwing an exception otherwise.
	*
	* @param string $sApiKey
	* @return bool
	* @throws Vidlinkr_Exception
	*/
	private static function _verifyPrivateApiKey ($sApiKey)
	{

		$oResponse = Vidlinkr_Api::doRequest('open/login', array('private_api_key'=>$sApiKey));

		if (Vidlinkr_Api::STATUS_BAD_REQUEST == $oResponse->status->code) {
			throw new Vidlinkr_Exception('The API key provided is not valid');
		}

		if (Vidlinkr_Api::STATUS_OK != $oResponse->status->code) {
			throw new Vidlinkr_Exception(Vidlinkr_Api::getResponseMessage($oResponse));
		}

		return true;

	}

	private static function _getPrivateApiKeyFromLogin ($sEmail, $sPassword)
	{

		$oResponse = Vidlinkr_Api::doRequest('open/login', array('email'=>$sEmail,'password'=>$sPassword));

		if (Vidlinkr_Api::STATUS_BAD_REQUEST == $oResponse->status->code) {
			throw new Vidlinkr_Exception('The login details provided are not valid');
		}

		if (Vidlinkr_Api::STATUS_OK != $oResponse->status->code) {
			throw new Vidlinkr_Exception(Vidlinkr_Api::getResponseMessage($oResponse));
		}

		if (false == isset($oResponse->data->session->token)) {
			throw new Vidlinkr_Exception('Login did not return a session token');
		}

		$oResponse = Vidlinkr_Api::doRequest('account/getprivateapikey', array('session_token'=>$oResponse->data->session->token));

		if (Vidlinkr_Api::STATUS_OK != $oResponse->status->code) {
			throw new Vidlinkr_Exception(Vidlinkr_Api::getResponseMessage($oResponse));
		}

		if (false == isset($oResponse->data->private_api_key)) {
			throw new Vidlinkr_Exception('Request did not return a private API key');
		}

		return $oResponse->data->private_api_key;

	}

	/**
	* Fetch the details for the website identified by $sUrl, adding them to the
	* local config and returning true if found, otherwise returning false.
	* or throwing an exception otherwise.
	*
	* @param string $sUrl
	* @return bool
	* @throws Vidlinkr_Exception
	*/
	private static function _getWebsite ($sUrl)
	{

		$oResponse = Vidlinkr_Api::doRequest('account/getwebsite', array(
			'private_api_key'	=> self::getConfig(self::CONFIG_KEY_PRIVATE_API_KEY),
			'url'				=> $sUrl
		));

		if (Vidlinkr_Api::STATUS_OK == $oResponse->status->code) {

			self::setConfig(self::CONFIG_KEY_WEBSITE_ID, $oResponse->data->website->id);
			self::setConfig(self::CONFIG_KEY_WEBSITE_URL, $oResponse->data->website->url);
			self::setConfig(self::CONFIG_KEY_PUBLISHER_ID, $oResponse->data->website->account_id);

			return true;
		}

		if (Vidlinkr_Api::STATUS_NOT_FOUND == $oResponse->status->code) {
			return false;
		}

		throw new Vidlinkr_Exception(Vidlinkr_Api::getResponseMessage($oResponse));

	}

	/**
	* Add the website identified by $sUrl to, returning true if all went
	* well, otherwise throwing an exception.
	*
	* @param string $sUrl
	* @return bool
	* @throws Vidlinkr_Exception
	*/
	private static function _addWebsite ($sUrl)
	{

		$oResponse = Vidlinkr_Api::doRequest('account/addwebsite', array(
			'private_api_key'	=> self::getConfig(self::CONFIG_KEY_PRIVATE_API_KEY),
			'url'				=> $sUrl
		));

		if (Vidlinkr_Api::STATUS_OK != $oResponse->status->code) {
			throw new Vidlinkr_Exception(Vidlinkr_Api::getResponseMessage($oResponse));
		}

		self::setConfig(self::CONFIG_KEY_WEBSITE_ID, $oResponse->data->website->id);
		self::setConfig(self::CONFIG_KEY_WEBSITE_URL, $oResponse->data->website->url);
		self::setConfig(self::CONFIG_KEY_PUBLISHER_ID, $oResponse->data->website->account_id);

		return true;

	}

	/**
	* Return an item from the config. If $sKey is not specified, return
	* the config array, otherwise return the item identified by $sKey,
	* or null if it is not set.
	*
	* @param string $sKey
	* @return mixed
	*/
	public static function getConfig ($sKey=null)
	{

		if (null == self::$_aConfig) {
			self::$_aConfig = (array) get_option(self::WP_OPTION_CONFIG);
		}

		if (null == $sKey) {
			return self::$_aConfig;
		}

		if (false == isset(self::$_aConfig[$sKey])) {
			return null;
		}

		return self::$_aConfig[$sKey];

	}

	/**
	* Set a config item
	*
	* @param string $sKey
	* @param string $sVal
	*/
	public static function setConfig ($sKey, $sVal)
	{

		if (null === self::$_aConfig) {
			self::getConfig();
		}

		self::$_aConfig[$sKey] = $sVal;

	}

	/**
	* Store the config in the WordPress DB, throwing an exception
	* if an error occurs.
	*
	* @throws Vidlinkr_Exception
	*/
	public static function storeConfig ()
	{

		if (false == is_array(self::$_aConfig)) {
			throw new Vidlinkr_Exception('Attempt to store uninitialized config');
		}

		if (false == update_option(self::WP_OPTION_CONFIG, self::$_aConfig)) {
			throw new Vidlinkr_Exception('Config could not be stored');
		}

	}

	/**
	* Wrapper for individual methods for each provider, testing that
	* publisher id and website id are both set before making changes.
	*
	* @param string $sHtml
	* @return string
	*/
	public static function swapUrls ($sHtml)
	{

		/**
		* If either publisher id or website id is null, return the content unchanged
		*/
		if (null == self::getConfig(self::CONFIG_KEY_PUBLISHER_ID) || null == self::getConfig(self::CONFIG_KEY_WEBSITE_ID)) {
			return $sHtml;
		}

		return self::_swapYouTubeUrls($sHtml);
	}

	/**
	* Swap out YouTube URLs
	*
	* @param string $sHtml
	* @return string
	*/
	private static function _swapYouTubeUrls ($sHtml)
	{

		static $sRegexVideoId = '[0-9A-Za-z_-]+';
		static $sRegexMainHost = 'https?://(?:www\.)?youtube\.com';
		static $sRegexShortHost = 'https?://(?:www\.)?youtu\.be';

		static $sRegexIframe = '(<iframe[^>]+)';

		$sApiUrlYouTube = sprintf('http://%s/activatevideo?video_provider_id=%d&pid=%d&website_id=%d&video_provider_url=',
			Vidlinkr_Api::API_HOST,
			Vidlinkr_Api::VIDEO_PROVIDER_ID_YOUTUBE,
			self::getConfig(self::CONFIG_KEY_PUBLISHER_ID),
			self::getConfig(self::CONFIG_KEY_WEBSITE_ID)
		);

		$sEvalString = '"$1".$sApiUrlYouTube.urlencode("$2")';

		// e.g. http://www.youtube.com/watch?v=Js7HYv-14v8&feature=xxx
		$sHtml = preg_replace('#' . $sRegexIframe . '(' . $sRegexMainHost . '/watch?.*v=(' . $sRegexVideoId . '))#ie', $sEvalString, $sHtml);

		// e.g. http://www.youtube.com/embed/Js7HYv-14v8
		// e.g. http://www.youtube.com/v/Js7HYv-14v8
		$sHtml = preg_replace('#' . $sRegexIframe . '(' . $sRegexMainHost . '/(?:embed|v)/(' . $sRegexVideoId . '))#ie', $sEvalString, $sHtml);

		// e.g. http://youtu.be/Js7HYv-14v8
		$sHtml = preg_replace('#' . $sRegexIframe . '(' . $sRegexShortHost . '/(' . $sRegexVideoId . '))#ie', $sEvalString, $sHtml);

		return $sHtml;

	}

	/**
	* Obscure an API key, leaving only the first and last three characters visible.
	*/
	public function getObscuredApiKey ($sPrivateApiKey)
	{
		return preg_replace('#^(.{3})(.*)(.{3})$#e', "$1.str_repeat('*',strlen('$2')).$3", $sPrivateApiKey);
	}

	private static function _setErrorMessage ($sMessage=null)
	{
		self::$_sErrorMessage = $sMessage;
	}

	public static function getErrorMessage ()
	{
		return self::$_sErrorMessage;
	}

	private static function _setSuccessMessage ($sMessage=null)
	{
		self::$_sSuccessMessage = $sMessage;
	}

	public static function getSuccessMessage ()
	{
		return self::$_sSuccessMessage;
	}

}
